"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var t = require("babel-types");

exports.default = function () {
    return {
        visitor: {
            ForInStatement: {
                enter: ForInStatement
            },
            ForOfStatement: {
                enter: ForOfStatement
            }
        }
    };
};

function ForOfStatement(path) {

    if (path.node.expressive_binds) return;

    var _parseBindings = parseBindings(path),
        source = _parseBindings.source,
        bindings = _parseBindings.bindings,
        _parseBindings$kind = _parseBindings.kind,
        kind = _parseBindings$kind === undefined ? "const" : _parseBindings$kind;

    var _objectIterated = source.node,
        _itemReference = void 0,
        _itemKey = void 0,
        _itemDestructure = void 0;

    switch (bindings.length) {
        case 3:
            var _bindings = _slicedToArray(bindings, 3);

            _itemDestructure = _bindings[0];
            _itemReference = _bindings[1];
            _itemKey = _bindings[2];

            break;
        default:
            var _bindings2 = _slicedToArray(bindings, 2);

            _itemReference = _bindings2[0];
            _itemKey = _bindings2[1];

    }

    switch (_objectIterated.type) {
        case "StringLiteral":
            _objectIterated = t.arrayExpression(_objectIterated.value.split(/, +/).map(function (e) {
                return t.stringLiteral(e);
            }));
            break;

        case "TemplateLiteral":
            _objectIterated = t.callExpression(t.memberExpression(_objectIterated, t.identifier("split")), [t.regExpLiteral(", +")]);
            break;

        case "NumericLiteral":
            {
                var number = _objectIterated.value;
                if (!Number.isInteger(number)) bind.source.buildCodeFrameError("For range must be an integer");
                _objectIterated = t.callExpression(t.memberExpression(t.callExpression(t.identifier("Array"), [_objectIterated]), t.identifier("keys")), []);
            }break;
    }

    var output = void 0;
    var stats = extractStatements(path.get("body"));

    if (!_itemKey) {
        output = t.forOfStatement(t.variableDeclaration(kind, [t.variableDeclarator(_itemReference)]), _objectIterated, t.blockStatement(stats));
    } else {
        var init = [t.variableDeclarator(_itemKey, t.numericLiteral(0))];
        if (_objectIterated.type != "Identifier") {
            var _source = _objectIterated;
            _objectIterated = path.scope.generateUidIdentifier("obj");
            init.push(t.variableDeclarator(_objectIterated, _source));
        }
        output = t.forStatement(t.variableDeclaration("let", init), t.binaryExpression("<", _itemKey, t.memberExpression(_objectIterated, t.identifier("length"))), t.updateExpression("++", _itemKey), t.blockStatement(stats));

        var _bind = [t.variableDeclarator(_itemReference, t.memberExpression(_objectIterated, _itemKey, true))];
        if (_itemDestructure) _bind.push(t.variableDeclarator(_itemDestructure, _itemReference));
        stats.unshift(t.variableDeclaration(kind, _bind));
    }

    output.expressive_binds = bindings;

    path.replaceWith(output);
}

function ForInStatement(path) {

    if (path.node.expressive_binds) return;

    var bind = parseBindings(path);
    var _source = bind.source.node,
        bindings = bind.bindings,
        _bind$kind = bind.kind,
        kind = _bind$kind === undefined ? "const" : _bind$kind;


    if (~["StringLiteral", "TemplateLiteral"].indexOf(_source.type)) {
        throw path.get("right").buildCodeFrameError("A string here will cause runtime errors. Did you mean to use `for of` instead?");
    }

    var initial_bindings = [];

    var _key = void 0,
        _value = void 0,
        _struct = void 0;

    switch (bindings.length) {
        case 3:
            var _bindings3 = _slicedToArray(bindings, 3);

            _struct = _bindings3[0];
            _value = _bindings3[1];
            _key = _bindings3[2];

            break;
        case 2:
            var _bindings4 = _slicedToArray(bindings, 2);

            _value = _bindings4[0];
            _key = _bindings4[1];

            break;
        case 1:
            var _bindings5 = _slicedToArray(bindings, 1);

            _key = _bindings5[0];

            if (t.isObjectPattern(_key)) {
                _struct = _key;
                _key = path.scope.generateUidIdentifier("_key");
            }
            break;
    }

    var stats = extractStatements(path.get("body"));

    if (_value || _struct) {
        var _initial_bindings = [];

        if (!_value) {
            _initial_bindings.push(t.variableDeclarator(_struct, t.memberExpression(_source, _key, true)));
        } else {
            _initial_bindings.push(t.variableDeclarator(_value, t.memberExpression(_source, _key, true)));
            if (_struct) _initial_bindings.push(t.variableDeclarator(_struct, _value));
        }

        stats.unshift(t.variableDeclaration(kind, _initial_bindings));
    }

    var loop = t.forInStatement(t.variableDeclaration(kind, [t.variableDeclarator(_key)]), _source, t.blockStatement(stats));

    loop.expressive_binds = bind;

    path.replaceWith(loop);
}

function extractStatements(body) {
    if (body.isBlockStatement()) return body.node.body;else return [body.node];
}

function parseBindings(path) {

    var bindings = [];
    var kind = void 0;

    var iterable = path.get("right");

    if (iterable.isBinaryExpression({ operator: "in" })) {
        var extra = iterable.get("left");
        iterable = iterable.get("right");
        while (extra.isBinaryExpression({ operator: "in" })) {
            bindings.push(extra.get("right"));
            extra = extra.get("left");
        }
        bindings.push(extra);
    }

    var left = path.get("left");
    if (left.isVariableDeclaration()) {
        kind = left.node.kind;
        left = left.get("declarations.0.id");
    }

    bindings.push(left);
    bindings = bindings.reverse();

    if (bindings[2]) {
        if (!bindings[0].isObjectPattern()) {
            // debugger
            // throw bindings[0].buildCodeFrameError("Malformed arguments: expected ObjectPattern")
        }
        if (bindings[3]) throw bindings[3].buildCodeFrameError("Malformed arguments: too many arguments!");
    }

    return {
        source: iterable,
        bindings: bindings.map(function (x) {
            return x.node;
        }),
        kind: kind
    };
}